Scriptname _mz_HT_ActorScript extends activemagiceffect  
;Version 3.0

bool property RefreshHelmetBusy = false auto hidden
Actor property ActorRef auto hidden
Armor[] property HelmetList auto hidden
Bool[] property HelmetToggleState auto hidden
Int [] property HelmetMCMOption auto hidden
_mz_HT_MCMScript property MCMScript auto
int[] HexadecimalHelmetSlotMask
int[] DecimalHelmetSlotMasks

;Helmet Keywords
;------------------------------------------------
Keyword property ArmorHelmet auto
Keyword property ClothingHead auto
;------------------------------------------------

Event OnEffectStart(Actor akTarget, Actor akCaster)
	ActorRef = akCaster
	RegisterForNiNodeUpdate()
	
	HexadecimalHelmetSlotMask = new int [5]
		HexadecimalHelmetSlotMask[0] = 0x00000001 ;HEAD
		HexadecimalHelmetSlotMask[1] = 0x00000002 ;Hair
		HexadecimalHelmetSlotMask[2] = 0x00000800 ;LongHair
		HexadecimalHelmetSlotMask[3] = 0x00001000 ;Circlet
		HexadecimalHelmetSlotMask[4] = 0x00002000 ;Ears
	DecimalHelmetSlotMasks = new int[5]
		DecimalHelmetSlotMasks[0] = 1 ;HEAD
		DecimalHelmetSlotMasks[1] = 2 ;Hair
		DecimalHelmetSlotMasks[2] = 2048 ;LongHair
		DecimalHelmetSlotMasks[3] = 4096 ;Circlet
		DecimalHelmetSlotMasks[4] = 8192 ;Ears
	HelmetList = new Armor[5]
	HelmetToggleState = new Bool[5]
	HelmetMCMOption = new Int[5]
	
	int EmptyIndex = GetEmptyElementIndex(MCMScript.ActorList)
	if (EmptyIndex != -1)
		debug.trace("Actor list index " + EmptyIndex + " = " + self)
		MCMScript.ActorList[EmptyIndex] = self
	endif
	CompileHelmetList()
endEvent

Event OnEffectFinish(Actor akTarget, Actor akCaster)
	if !(akTarget.IsOnMount())
		akTarget.QueueNiNodeUpdate()
	endif
endEvent

int Function GetEmptyElementIndex(_mz_HT_ActorScript[] ScriptArray)
	int i = 0
	while (i < ScriptArray.length)
		if (ScriptArray[i] == None)
			return i
		else
			i += 1
		endif
	endWhile
	return -1
endFunction

int[] Function GetSlotMasks(int SlotMask)
	int[] SlotMaskList = new int[32]
	int CheckedSlotMasks = 0
	int SlotMaskIndex = 0
	while (SlotMask - CheckedSlotMasks != 0)
		SlotMaskList[SlotMaskIndex] = GetGreatestPowOf2(SlotMask - CheckedSlotMasks)
		CheckedSlotMasks += SlotMaskList[SlotMaskIndex]
		SlotMaskIndex += 1
	endWhile
	return SlotMaskList
endFunction

int Function GetGreatestPowOf2(int IntToCheck)
	int i = 1
	int Lasti = 0
	while (i <= IntToCheck)
		Lasti = i
		i*=2
	endwhile
	return Lasti
endFunction


Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)	
	if bIsHelmet(akBaseObject)
		debug.trace(akBaseObject.GetName() + " equipped")
		
		int[] DecimalSlotMasks = new int[32]
		DecimalSlotMasks = GetSlotMasks((akBaseObject as armor).GetSlotMask())
		
		int i = 0
		while (i < 5)
			if (HelmetList[i] != None && HelmetList[i] != akBaseObject && ActorRef.IsEquipped(HelmetList[i]) && DecimalSlotMasks.find(DecimalHelmetSlotMasks[i]) > -1)
				;debug.Trace("Found Helmet equipped in slot " + i + " . Slot masks = " + DecimalSlotMasks + " found " + DecimalHelmetSlotMasks[i])
				debug.trace(HelmetList[i].GetName() +  " in Slot " + i + " Conflicts with " + akBaseObject.GetName() + ". Slot Conflict = " + DecimalHelmetSlotMasks[i])
				ActorRef.Unequipitem(HelmetList[i])
				HelmetList[i] = None
				HelmetToggleState[i] = false
			endif
			i += 1
		endwhile
	endif	
	if (RefreshHelmetBusy == false && !ActorRef.IsOnMount())
		RefreshHelmetBusy = true
		RefreshHelmets()
	endif
endEvent

Event OnObjectUnequipped(Form akBaseObject, ObjectReference akReference)
	if (RefreshHelmetBusy == false && !ActorRef.IsOnMount())
		RefreshHelmetBusy = true
		RefreshHelmets()
	endif
endevent

function RefreshHelmets()
	if ActorRef.IsOnMount()
		debug.notification("Helmet Toggle - Cannot Refresh Helmet While Mounted.")
		utility.wait(0.1)
		RefreshHelmetBusy = false
		return
	endif
	
	debug.trace("Refreshing Helmets " + self)
	Armor[] HelmetstoHideList = new Armor[5]
	Armor[] TempHelmetList = new Armor[5]
	int[] HelmetToHideSlotMaskList = new int[5]
	int i = 0
	while (i < 5)
		armor HelmetToCheck = HelmetList[i]
		if (HelmetToCheck != none && HelmetToggleState[i] == true && ActorRef.IsEquipped(HelmetToCheck) && TempHelmetList.find(HelmetToCheck) < 0)
			debug.trace("Refreshing " + HelmetToCheck.GetName() + " on Actor " + ActorRef.GetActorBase().GetName())
			TempHelmetList[i] = HelmetToCheck
			HelmetstoHideList[i] = HelmetToCheck
			HelmetToHideSlotMaskList[i] = HelmetToCheck.GetSlotMask()	
			HelmetToCheck.RemoveSlotFromMask(0x00003803)
		endif
		i += 1
	endWhile
	utility.WaitMenuMode(MCMScript.RefreshDelay)
	ActorRef.QueueNiNodeUpdate()
	utility.WaitMenuMode(MCMScript.RefreshDelay)
	i = 0
	while (i < 5)
		if (HelmetstoHideList[i] != None)
			HelmetstoHideList[i].SetSlotMask(HelmetToHideSlotMaskList[i])
		endif
		i+= 1
	endwhile
	utility.WaitMenuMode(0.1)
	RefreshHelmetBusy = false
endFunction

	
	bool Function bIsHelmet(form akBaseObject)
	if akBaseObject.GetType() == 26 && (akBaseObject.hasKeyword(ArmorHelmet) || akBaseObject.hasKeyword(ClothingHead))
		return true
	else
		return false
	endif
endFunction

Function CompileHelmetList()
	if (ActorRef == None)
		return
	endif
	debug.trace("Helmet Toggle Actor Script - Compiling Actor Helmet List : " + ActorRef.GetActorBase().GetName())
	
	HelmetList = new Armor[5]
	
	;Fills List with any armors equipped by the player in any of the Helmet slots.
	int i = 0
	while (i < 5)
		Armor HelmetInSlot = ActorRef.GetWornForm(HexadecimalHelmetSlotMask[i]) as armor
		if (HelmetInSlot != None)
			HelmetList[i] = HelmetInSlot
		endif
		i += 1
	endWhile
endFunction